/* (c) 2007 Satis Superque Merce B.V. All rights reserved.
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 */

#include <b.h>

b_error_t
b_triple_check (b_triple_t * triple)
{

  B_LOG1 ("b_triple_check: %p", triple);

  if (!triple)
    return B_ERROR_DATA;

  if (!triple->subject_uri && !triple->subject_bnode)
    return B_ERROR_DATA;

  if (!triple->property)
    return B_ERROR_DATA;

  if (!triple->object_uri && !triple->object_bnode && !triple->object_literal)
    return B_ERROR_DATA;

  return B_OK;
}

/**
 * b_triple_new_incomplete:
 * @triple: The return pointer for the new b_triple_t data struct.
 * @subject_uri: The subject URI. It can be NULL if the subject_object is not 
 * NULL.
 * @subject_uri_len: The length of the subject_uri. Can be 0.
 * @subject_bnode: The subject BNODE.
 * @subject_bnode_len: The length of the subject_bnode. Can be 0.
 * @property: The property of this triple.
 * @property_len: The length of the property. Can be 0.
 * @object_uri: The object as URI. It can be NULL if the triple has not a URI
 * object.
 * @object_uri_len: The length of the URI object. Can be 0.
 * @object_bnode: The object as BNODE. It can be NULL if the triple has not a
 * BNODE object.
 * @object_bnode_len: The length of the BNODE object. Can be 0.
 * @object_literal: The object as Literal. It can be NULL if the triple has not
 * a Literal object.
 * @object_literal_len: The length of the Literal Object. Can be 0.
 * @context: The context. Can be NULL.
 * @context_len: The Context Length.
 * @datatype: The datatype for this Object Literal. It can be NULL.
 * @datatype_len: The length of the datatype.
 * @lang: The lang of this Object Literal.
 * @lang_len: The length of the lang.
 * @returns: The error code.
 *
 * Creates a new triples. It will not checked so you can create a incomplete
 * triple for the iterator. This triple must be destroy with
 * b_triple_destroy().
 **/
b_error_t
b_triple_new_incomplete (b_triple_t ** ret, unsigned char *subject_uri,
			 b_uint64 subject_uri_len,
			 unsigned char *subject_bnode,
			 b_uint64 subject_bnode_len, unsigned char *property,
			 b_uint64 property_len, unsigned char *object_uri,
			 b_uint64 object_uri_len, unsigned char *object_bnode,
			 b_uint64 object_bnode_len,
			 unsigned char *object_literal,
			 b_uint64 object_literal_len, unsigned char *context,
			 b_uint64 context_len, unsigned char *datatype,
			 b_uint64 datatype_len, unsigned char *lang,
			 b_uint64 lang_len)
{
  b_triple_t *triple;

  B_LOG19
    ("b_triple_new_incomplete: %p %p %d %p %d %p %d %p %d %p %d %p %d %p %d %p %d %p %d",
     ret, subject_uri, (int) subject_uri_len, subject_bnode,
     (int) subject_bnode_len, property, (int) property_len, object_uri,
     (int) object_uri_len, object_bnode, (int) object_bnode_len,
     object_literal, (int) object_literal_len, context, (int) context_len,
     datatype, (int) datatype_len, lang, (int) lang_len);

  if (!ret)
    return B_ERROR_DATA;

  if (!(triple = (b_triple_t *) calloc (1, sizeof (b_triple_t))))
    return B_ERROR_MEMORY;

#define B_TRIPLE_NEW( x, len ) \
  if(x) \
    { \
      triple->len = len > 0 ? len : strlen ((char *)x); \
      if (!(triple->x = b_string_alloc (x, triple->len))) \
        { \
          b_triple_destroy (triple); \
          return B_ERROR_MEMORY; \
        } \
    }

  B_TRIPLE_NEW (subject_uri, subject_uri_len);
  B_TRIPLE_NEW (subject_bnode, subject_bnode_len);
  B_TRIPLE_NEW (property, property_len);
  B_TRIPLE_NEW (object_uri, object_uri_len);
  B_TRIPLE_NEW (object_bnode, object_bnode_len);
  B_TRIPLE_NEW (object_literal, object_literal_len);
  B_TRIPLE_NEW (context, context_len);
  B_TRIPLE_NEW (datatype, datatype_len);
  B_TRIPLE_NEW (lang, lang_len);

  *ret = triple;
  return B_OK;
}

/**
 * b_triple_new:
 * @triple: The return pointer for the new b_triple_t data struct.
 * @subject_uri: The subject URI. It can be NULL if the subject_object is not 
 * NULL.
 * @subject_uri_len: The length of the subject_uri. Can be 0.
 * @subject_bnode: The subject BNODE.
 * @subject_bnode_len: The length of the subject_bnode. Can be 0.
 * @property: The property of this triple.
 * @property_len: The length of the property. Can be 0.
 * @object_uri: The object as URI. It can be NULL if the triple has not a URI
 * object.
 * @object_uri_len: The length of the URI object. Can be 0.
 * @object_bnode: The object as BNODE. It can be NULL if the triple has not a
 * BNODE object.
 * @object_bnode_len: The length of the BNODE object. Can be 0.
 * @object_literal: The object as Literal. It can be NULL if the triple has not
 * a Literal object.
 * @object_literal_len: The length of the Literal Object. Can be 0.
 * @context: The context. Can be NULL.
 * @context_len: The Context Length.
 * @datatype: The datatype for this Object Literal. It can be NULL.
 * @datatype_len: The length of the datatype.
 * @lang: The lang of this Object Literal.
 * @lang_len: The length of the lang.
 * @returns: The error code.
 *
 * Creates a new triples. This triple must be destroy with b_triple_destroy().
 **/
b_error_t
b_triple_new (b_triple_t ** ret, unsigned char *subject_uri,
	      b_uint64 subject_uri_len, unsigned char *subject_bnode,
	      b_uint64 subject_bnode_len, unsigned char *property,
	      b_uint64 property_len, unsigned char *object_uri,
	      b_uint64 object_uri_len, unsigned char *object_bnode,
	      b_uint64 object_bnode_len, unsigned char *object_literal,
	      b_uint64 object_literal_len, unsigned char *context,
	      b_uint64 context_len, unsigned char *datatype,
	      b_uint64 datatype_len, unsigned char *lang, b_uint64 lang_len)
{
  b_error_t error;

  B_LOG19
    ("b_triple_new: %p %p %d %p %d %p %d %p %d %p %d %p %d %p %d %p %d %p %d",
     ret, subject_uri, (int) subject_uri_len, subject_bnode,
     (int) subject_bnode_len, property, (int) property_len, object_uri,
     (int) object_uri_len, object_bnode, (int) object_bnode_len,
     object_literal, (int) object_literal_len, context, (int) context_len,
     datatype, (int) datatype_len, lang, (int) lang_len);

  if ((error =
       b_triple_new_incomplete (ret, subject_uri, subject_uri_len,
				subject_bnode, subject_bnode_len, property,
				property_len, object_uri, object_uri_len,
				object_bnode, object_bnode_len,
				object_literal, object_literal_len, context,
				context_len, datatype, datatype_len, lang,
				lang_len)) != B_OK)
    return error;

  if (b_triple_check (*ret) != B_OK)
    {
      b_triple_destroy (*ret);
      return B_ERROR_DATA;
    }

  return B_OK;
}

b_error_t
b_triple_new_cache (b_tcache_t * cache, b_triple_t ** ret)
{
  b_triple_t *triple;

  B_LOG2 ("b_triple_new_cache: %p %p", cache, ret);

  if (!ret || !cache)
    return B_ERROR_DATA;

  if (!(triple = (b_triple_t *) calloc (1, sizeof (b_triple_t))))
    return B_ERROR_MEMORY;

#define B_TRIPLE_NEW_CACHE( x , l ) \
  if (cache->x) \
    { \
      triple->x = (unsigned char *)strdup ((char *)cache->x->what); \
      triple->l = cache->x->len; \
    }

  B_TRIPLE_NEW_CACHE (subject_uri, subject_uri_len);
  B_TRIPLE_NEW_CACHE (subject_bnode, subject_bnode_len);
  B_TRIPLE_NEW_CACHE (property, property_len);
  B_TRIPLE_NEW_CACHE (object_uri, object_uri_len);
  B_TRIPLE_NEW_CACHE (object_bnode, object_bnode_len);
  B_TRIPLE_NEW_CACHE (object_literal, object_literal_len);
  B_TRIPLE_NEW_CACHE (context, context_len);
  B_TRIPLE_NEW_CACHE (datatype, datatype_len);
  B_TRIPLE_NEW_CACHE (lang, lang_len);

  *ret = triple;
  return B_OK;
}

/**
 * b_triple_destroy:
 * @triple: The Pointer to the b_triple_t data struct.
 * @returns: The error code.
 *
 * Destroies the triple data struct.
 **/
b_error_t
b_triple_destroy (b_triple_t * t)
{
  B_LOG1 ("b_triple_destroy: %p", t);

  if (!t)
    return B_ERROR_DATA;

  if (t->subject_uri)
    free (t->subject_uri);

  if (t->subject_bnode)
    free (t->subject_bnode);

  if (t->property)
    free (t->property);

  if (t->object_uri)
    free (t->object_uri);

  if (t->object_bnode)
    free (t->object_bnode);

  if (t->object_literal)
    free (t->object_literal);

  if (t->context)
    free (t->context);

  if (t->datatype)
    free (t->datatype);

  if (t->lang)
    free (t->lang);

  free (t);

  return B_OK;
}

/**
 * b_triple_print:
 * @file: The FILE.
 * @triple: The b_triple_t pointer.
 * @returns: The error code.
 *
 * Prints the b_triple_t in n3 syntax in a FILE *.
 **/
b_error_t
b_triple_print (FILE * file, b_triple_t * triple)
{
  B_LOG2 ("b_triple_print: %p %p", file, triple);

  if (!file || !triple)
    return B_ERROR_DATA;

  if (b_triple_check (triple) != B_OK)
    return B_ERROR_DATA;

  if (triple->subject_uri)
    fprintf (file, "<%s>", triple->subject_uri);
  else
    fprintf (file, "[%s]", triple->subject_bnode);

  fprintf (file, " <%s>", triple->property);

  if (triple->object_uri)
    fprintf (file, " <%s>", triple->object_uri);

  if (triple->object_bnode)
    fprintf (file, " [%s]", triple->object_bnode);

  if (triple->object_literal)
    fprintf (file, " \"%s\"", triple->object_literal);

  if (triple->lang)
    fprintf (file, "@%s", triple->lang);

  if (triple->datatype)
    fprintf (file, "^^<%s>", triple->datatype);

  if (triple->context)
    fprintf (file, " <%s>", triple->context);

  fprintf (file, "\n");

  return B_OK;
}

b_error_t
b_copy_triple (b_triple_t * src, b_triple_t ** dst)
{
  b_triple_t *new;

  B_LOG2 ("b_triple)copy: %p %p", src, dst);

  if (!src)
    return B_ERROR_DATA;

  if (!(new = (b_triple_t *) calloc (1, sizeof (b_triple_t))))
    return B_ERROR_MEMORY;

#define B_COPY_TRIPLE( x , len ) \
  if (src->x) \
    { \
      new->len = src->len > 0 ? src->len : strlen ((char *) src->x); \
      if (!(new->x = b_string_alloc (src->x, new->len))) \
	{ \
	  b_triple_destroy (new); \
	  return B_ERROR_MEMORY; \
	} \
    }

  B_COPY_TRIPLE (subject_uri, subject_uri_len);
  B_COPY_TRIPLE (subject_bnode, subject_bnode_len);
  B_COPY_TRIPLE (property, property_len);
  B_COPY_TRIPLE (object_uri, object_uri_len);
  B_COPY_TRIPLE (object_bnode, object_bnode_len);
  B_COPY_TRIPLE (object_literal, object_literal_len);
  B_COPY_TRIPLE (context, context_len);
  B_COPY_TRIPLE (datatype, datatype_len);
  B_COPY_TRIPLE (lang, lang_len);

  *dst = new;
  return B_OK;
}

/* EOF */
