#include <b.h>

#define TEST3_MAX 4096

#ifdef WIN32
#define snprintf sprintf_s
#define TEST3_FILE ".\\TEST3"
#else
#define TEST3_FILE "/tmp/test3"
#endif

int
main (int argc, char **argv)
{
  b_t *b;
  char s[128];
  int i, c, m;
  b_error_t err;

  b_iterator_type_t *iterator;
  unsigned char *what;
  b_uint64 size;

  fprintf (stderr, "Creating new b struct... ");
  if ((err = b_new (&b, (unsigned char *) TEST3_FILE)) != B_OK)
    {
      fprintf (stderr, "b_new: %s\n", b_strerror (err));
      return 1;
    }

  if (argc > 1 && (c = atoi (argv[1])))
    {
      fprintf (stderr, "done.\nSetting cache to %d... ", c);
      b_set_max_cache (b, c);
    }

  if (argc <= 2 || !(m = atoi (argv[2])))
    m = TEST3_MAX;

  fprintf (stderr, "done.\nStoring %d datas...", m);

  for (i = 0; i < m; i++)
    {
      snprintf (s, sizeof (s), "test %d", i);

      if ((err =
	   b_add_type (b, B_TYPE_URI, (unsigned char *) s,
		       i % 2 ? 0 : strlen (s))) != B_OK)
	{
	  fprintf (stderr, "b_add: %s\n", b_strerror (err));
	  return 1;
	}
    }

  fprintf (stderr, "done.\nCreating a iterator type... ");
  if ((err = b_iterator_type_new (b, B_TYPE_URI, &iterator)) != B_OK)
    {
      fprintf (stderr, "b_iterator_new: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\n");

  while ((err = b_iterator_type_step (iterator, &what, &size)) == B_OK
	 && what)
    {
      fprintf (stderr, "Next Data: %s (%d)\n", what, (int) size);
      free (what);
    }

  if (err != B_OK)
    {
      fprintf (stderr, "b_iterator_step: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\nDestroying the iterator type... ");
  if ((err = b_iterator_type_destroy (iterator)) != B_OK)
    {
      fprintf (stderr, "b_iterator_destroy: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\nDestroing the b struct... ");

  if ((err = b_destroy (b)) != B_OK)
    {
      fprintf (stderr, "b_destroy: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\n");
  return 0;
}
